import 'dart:convert';

import 'package:http/http.dart';
import 'package:medium_search/src/users/users_page.dart';

import 'posts/posts_page.dart';
import 'posts/posts_search_dto.dart' as posts;
import 'publications/publications_page.dart';
import 'publications/publications_search_dto.dart';
import 'tags/tags_search_dto.dart' as tags;
import 'users/users_search_dto.dart';

class Medium {
  final Client _client;

  Medium([Client? client]) : _client = client ?? Client();

  /// Search for posts.
  /// [q] is the query.
  /// [page] page number.
  /// [count] amount of queries per page.
  /// [ignoredIds] the ids of users that you want to prevent.
  Future<PostsPage> getPosts(
    String q, {
    int page = 1,
    int count = 10,
    List<String>? ignoredIds,
  }) async {
    var url = "https://medium.com/search/posts?q=$q";

    var body = <String, dynamic>{
      "count": count.toString(),
      "pageSize": count.toString(), // just to make sure.
    };

    if (page > 1) {
      body.putIfAbsent("page", () => page.toString());
    }

    if (ignoredIds != null) {
      body.putIfAbsent("ignoredIds", () => ignoredIds);
    }

    var resp = await _client.post(
      Uri.parse(url),
      headers: {
        "accept": "application/json",
        "x-xsrf-token": 1.toString(),
        "content-type": "application/json",
      },
      body: jsonEncode(body),
    );

    return PostsPage(
      page,
      q,
      count,
      this,
      posts.PostsSearchDto.fromJson(
        jsonDecode(
          resp.body.replaceFirst("])}while(1);</x>", ""),
        ),
      ),
    );
  }

  /// Search for publications.
  /// [q] is the query.
  /// [page] page number.
  /// [count] amount of queries per page.
  /// [ignoredIds] the ids of users that you want to prevent.
  Future<PublicationsPage> getPublications(
    String q, {
    int page = 1,
    int count = 10,
    List<String>? ignoredIds,
  }) async {
    var url = "https://medium.com/search/publications?q=$q";

    var body = <String, dynamic>{
      "count": count.toString(),
      "pageSize": count.toString(), // just to make sure.
    };

    if (page > 1) {
      body.putIfAbsent("page", () => page.toString());
    }

    if (ignoredIds != null) {
      body.putIfAbsent("ignoredIds", () => ignoredIds);
    }

    var resp = await _client.post(
      Uri.parse(url),
      headers: {
        "accept": "application/json",
        "x-xsrf-token": 1.toString(),
        "content-type": "application/json",
      },
      body: jsonEncode(body),
    );

    return PublicationsPage(
      page,
      q,
      count,
      this,
      PublicationsSearchDto.fromJson(
        jsonDecode(
          resp.body.replaceFirst("])}while(1);</x>", ""),
        ),
      ),
    );
  }

  /// Search for tags.
  /// [q] is the query.
  /// [count] the amount of items you want to get. (it seems that the api doesn't really respect this parameter)
  Future<List<tags.Tag>> getTags(String q, [int count = 10]) async {
    var url = "https://medium.com/search/tags";

    var resp = await _client.post(
      Uri.parse(url),
      headers: {
        "accept": "application/json",
        "x-xsrf-token": 1.toString(),
      },
      body: {
        "q": q,
        "count": count.toString(),
      },
    );

    return tags.TagsSearchDto.fromJson(
            jsonDecode(resp.body.replaceFirst("])}while(1);</x>", "")))
        .tags;
  }

  /// Search for users.
  /// [q] is the query.
  /// [page] page number.
  /// [count] amount of queries per page.
  /// [ignoredIds] the ids of users that you want to prevent.
  Future<UsersPage> getUsers(
    String q, {
    int page = 1,
    int count = 10,
    List<String>? ignoredIds,
  }) async {
    var url = "https://medium.com/search/users?q=$q";

    var body = <String, dynamic>{
      "count": count.toString(),
      "pageSize": count.toString(), // just to make sure.
    };

    if (page > 1) {
      body.putIfAbsent("page", () => page.toString());
    }

    if (ignoredIds != null) {
      body.putIfAbsent("ignoredIds", () => ignoredIds);
    }

    var resp = await _client.post(
      Uri.parse(url),
      headers: {
        "accept": "application/json",
        "x-xsrf-token": 1.toString(),
        "content-type": "application/json",
      },
      body: jsonEncode(body),
    );

    return UsersPage(
      page,
      q,
      count,
      this,
      UsersSearchDto.fromJson(
        jsonDecode(
          resp.body.replaceFirst("])}while(1);</x>", ""),
        ),
      ),
    );
  }
}
